/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2000-2002. All Rights Reserved. 

Contributor(s): 
*/

package com.simonstl.ool;


import org.xml.sax.*;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.XMLFilter;
import org.xml.sax.helpers.XMLFilterImpl;

import com.megginson.sax.XMLWriter;
import com.megginson.sax.DataWriter;

import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.ParserAdapter;
import org.xml.sax.helpers.XMLReaderFactory;

import java.util.*;
import java.io.*;



/**
 * <p>This class wraps mixed  content textual fragments in mix:ed elements.</p>
 * 
 * 
 * 
 * @author Simon St.Laurent
 * @version 0.01 $Date: 2002/05/06 $
 */
public class MixedWrapSAXFilter extends XMLFilterImpl  
{

   protected boolean previousEndEvent=false;
   protected boolean ignoreWhitespace=true;
   protected StringBuffer chars=new StringBuffer();
   protected AttributesImpl emptyAtts=new AttributesImpl();
 
/**
 Set whether or not to ignore whitespace
*/
public void setIgnoreWhitespace(boolean _ignoreWhitespace) {
   ignoreWhitespace=_ignoreWhitespace;
}

/**
 Get whether or not to ignore whitespace
*/
public boolean getIgnoreWhitespace() {
   return ignoreWhitespace;
}
 
/**
* A constructor that takes the parser which will feed it SAX events
*/

public MixedWrapSAXFilter (XMLReader parent) {
      super(parent); 
} 

  /**
  Handles startElement events.
  */
public void startElement (String uri, String localName, String qName, Attributes atts) throws SAXException {
   
   //if the buffer has content, we're in mixed territory
     int charLength=chars.length();         
     
     if (charLength>0) {
       char[] temp=new char[charLength];
       chars.getChars(0, charLength, temp, 0);
       if (ignoreIt()) {
          super.characters(temp, 0, charLength);
          chars=new StringBuffer();
       } else {    
         super.startElement("http://simonstl.com/ns/mixed/", "mixed", "mix:ed", emptyAtts);
         super.characters(temp, 0, charLength);
         super.endElement("http://simonstl.com/ns/mixed/", "mixed", "mix:ed");
         chars=new StringBuffer();
       }
     }
   
   previousEndEvent=false;

   super.startElement(uri, localName, qName, atts);

}


  /**
  Handles characters events.
  */
public void characters (char[] ch, int start, int len)  throws SAXException {

      chars.append(ch, start, len);

//      super.characters(ch, start, len);
}

  /**
  Handles ignorableWhitespace events.
  */
public void ignorableWhitespace (char[] ch, int start, int len)  throws SAXException {
   if (!ignoreWhitespace) {
     characters(ch, start, len);
   } else {
     super.ignorableWhitespace(ch,start,len);
   }
}

  /**
  Handles endElement events.
  */
public void endElement (String uri, String localName, String qName) throws SAXException {

   int charLength=chars.length();
   
   //if the previous event was an end event, we need to check the buffer
   if (previousEndEvent) {
      //if the buffer has content, we're in mixed territory

     
     if (charLength>0) {
       char[] temp=new char[charLength];
       chars.getChars(0, charLength, temp, 0);
       if (ignoreIt()) {
          super.characters(temp, 0, charLength);
          chars=new StringBuffer();
       } else {    
         super.startElement("http://simonstl.com/ns/mixed/", "mixed", "mix:ed", emptyAtts);
         super.characters(temp, 0, charLength);
         super.endElement("http://simonstl.com/ns/mixed/", "mixed", "mix:ed");
         chars=new StringBuffer();
       }     }
   } else { //was just a normal element, pass it on.
       char[] temp=new char[charLength];
       chars.getChars(0, charLength, temp, 0);
       super.characters(temp, 0, charLength);
   
   }
   
   previousEndEvent=true;
   chars=new StringBuffer();

     super.endElement(uri, localName, qName);
}   



protected boolean ignoreIt(){
   int charLength=chars.length();
   for (int i=0;i<charLength;i++) {
     char testChar=chars.charAt(i);
     if (isWhitespace(testChar)){
     
     } else {
       return false;
     }
   }

   return ignoreWhitespace;
}

protected boolean isWhitespace(int testChar){
  boolean returnVal=false;
  
  if ( (testChar==0x0A) || (testChar==0x0D) || (testChar==0x20) || (testChar==0x09) ) {
   returnVal=true;

   }
//System.out.println("For char "+testChar+ " returned "+returnVal);

   return returnVal;
}



//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------

public static void printHelp() {
    System.out.println("Mixed requires two arguments - an origin file and a  destination file.");
    System.out.println("");
    System.out.println("Options may appear before those arguments:");
    System.out.println("-p [parser class name] - specifies a SAX2 parser for use with this tool");
    System.out.println("-v - verbose output");
    System.out.println("More information is available at http://simonstl.com/projects/ool/");
    System.out.println("");
/*
    System.out.println("");
    System.out.println("");
    System.out.println("");
    System.out.println("");
*/
}

/**
FilterTester's main method requires one argument - a target XML file.

Options may appear before those arguments:<br />
-p [parser class name]  - use the parser specified<br />


*/

  // Main Method
  public static void main (String[] args) throws Exception {

    int numArgs=args.length;

    String targetFileName;
    String originFileName;
    boolean suppressOol=true;

//deal with System properties
   if ((System.getProperty("org.xml.sax.driver"))==null) {
      System.setProperty("org.xml.sax.driver","org.apache.xerces.parsers.SAXParser");
   }


   String parserName=System.getProperty("org.xml.sax.driver");

    if (numArgs<2) {
      printHelp();
    } else {

       targetFileName=args[numArgs-1];
       originFileName=args[numArgs-2];

for (int i=0; i<numArgs; i++) {
   if (args[i].equals ("-p")) {
       parserName=args[i+1];
       i++;
   }
   if (args[i].equals ("-v")) {
       suppressOol=false;
   }

}
       //  Create files
       
       
       //  Create a SAX Parser
        XMLReader parser = XMLReaderFactory.createXMLReader(parserName); 
        
        MixedWrapSAXFilter mixedFilter=new MixedWrapSAXFilter(parser);


        FileWriter target=new FileWriter(targetFileName);  
        XMLWriter w = new XMLWriter(mixedFilter, target);
        FileReader origin=new FileReader(originFileName);
	    w.parse(new InputSource(origin));
      
      origin.close();
      target.close();


      
    }
  }


}
