/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2000-2002. All Rights Reserved. 

Contributor(s): 
*/

package com.simonstl.ool;


import org.xml.sax.*;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.XMLFilter;
import org.xml.sax.helpers.XMLFilterImpl;

import java.util.*;
import java.io.*;



/**
 * <p>This class is the first phase of converting XML documents into Oolish XML documents.  It replaces all character events with empty ool:text elements and diverts the text to a separate file.</p>
 * 
 * 
 * @author Simon St.Laurent
 * @version 0.01 $Date: 2002/04/12 $
 */
public class OolBaseSAXFilter extends XMLFilterImpl  
{
   protected Writer content;
   protected int currentPosition=0;
   protected boolean preserveWhitespace=false;

/**
* A constructor that takes the parser which will feed it SAX events
*/

   public OolBaseSAXFilter (XMLReader parent) {
      super(parent); 
   } 

/**
  Takes a writer into which all the text of the document will go.
*/
   public void setContent(Writer _content){
      content=_content;
   }   
   
/**
  Returns the Writer containing all the text.
*/   
   public Writer getContent(){
      return content;
   }
/**
   sets the current character position.  useful only if you've inserted or deleted text content.
*/  
   public void setCurrentPosition(int _position){
      currentPosition=_position;
   }
   /**
   gets the current character position.
*/  
   public int getCurrentPosition(){
      return currentPosition;
   }
   
  /**
   sets whether or not to preserve "ignorable whitespace", per SAX rules.
*/ 
   public void setPreserveWhitespace(boolean _preserve){
      preserveWhitespace=_preserve;
   }
   
   /**
   returns whether or not the filter is preserving "ignorable whitespace", per SAX rules.
*/ 
   public boolean getPreserveWhitespace(){
      return preserveWhitespace;
   }
   
   
   public void startElement (String uri, String localName, String qName, Attributes atts) throws SAXException {
     super.startElement(uri, localName, qName, atts);

   }

/**
   the characters event does the bulk of the work, sending text one way and elements (supplemented with Ool elements) another.
*/ 
public void characters (char[] ch, int start, int len)  throws SAXException {
            
      try {
        content.write(ch, start, len);
      } catch (IOException e) {
          SAXException ex=new SAXException("bad writer!",e);    
      }
      
      //create new element with attributes
      AttributesImpl oolAtts=new AttributesImpl();
      oolAtts.addAttribute("http://simonstl.com/ns/ool/", "start", "ool:start", "CDATA", String.valueOf(currentPosition));
      
      currentPosition=currentPosition+len;
      
      oolAtts.addAttribute("http://simonstl.com/ns/ool/", "end", "ool:start", "CDATA", String.valueOf(currentPosition));

       super.startPrefixMapping("ool","http://simonstl.com/ns/ool/");

        super.startElement("http://simonstl.com/ns/ool/", "text", "ool:text", oolAtts);
        super.endElement("http://simonstl.com/ns/ool/", "text", "ool:text");
        super.endPrefixMapping("ool");
}
   
   /**
   the ignorableWhitespace event sends its information the characters() method if whitespace is being preserved, and passes it up the SAX chain otherwise.
*/ 
      public void ignorableWhitespace (char[] ch, int start, int len)  throws SAXException {

      if (preserveWhitespace) {
         characters(ch, start, len);
      }

//      super.ignorableWhitespace (ch, start, len);
   }

}
