/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2000-2002. All Rights Reserved. 

Contributor(s): 
*/

package com.simonstl.ool;


import org.xml.sax.*;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.XMLFilter;
import org.xml.sax.helpers.XMLFilterImpl;

import com.megginson.sax.XMLWriter;
import com.megginson.sax.DataWriter;

import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.ParserAdapter;
import org.xml.sax.helpers.XMLReaderFactory;

import java.util.*;
import java.io.*;



/**
  * <p>This class removes single ool:text elements and puts their attributes into their parent element.</p>
 * <p><strong>Warning:</strong> This class is still very much under construction and is broken for most cases.</p>
 * <p><strong>Warning:</strong> this processor does not preserve the relative ordering of processing instructions intermingled with text.</p>
 * 
 * 
 * @author Simon St.Laurent
 * @version 0.01 $Date: 2002/05/06 $
 */
public class OolCleanSAXFilter extends XMLFilterImpl  
{

   protected boolean previousEndEvent=false;
   protected StartStack stack=new StartStack();
   protected StringBuffer chars=new StringBuffer();
   protected AttributesImpl emptyAtts=new AttributesImpl();
 
 
/**
* A constructor that takes the parser which will feed it SAX events
*/

public OolCleanSAXFilter (XMLReader parent) {
      super(parent); 
} 

  /**
  Handles startElement events.
  */
public void startElement (String uri, String localName, String qName, Attributes atts) throws SAXException {
System.out.println("Starting "+qName);
  if (uri.equals("http://simonstl.com/ns/ool/")){
    //ool element - put it on the stack
    StartTag tag=new StartTag(uri, localName, qName, atts);
    stack.push(tag);
//System.out.println("Adding to Stack length: "+stack.size());
  } else {
     //not an ool element 
   
     //if the stack has content, we're in mixed territory
     int stackLength=stack.size();         
     previousEndEvent=false;
     if (stackLength>0) {
       //read off the stack as a series of startElement events, followed by     
       //passing this startElement event
         reportStack(); 

//         reportStartElement(uri, localName, qName, atts);
     } 
         //add this event to the stack
     
         StartTag tag=new StartTag(uri, localName, qName, atts);
         stack.push(tag);
     
   
  }

}



  /**
  Handles endElement events.
  */
public void endElement (String uri, String localName, String qName) throws SAXException {
System.out.println("Ending "+qName);
  if (uri.equals("http://simonstl.com/ns/ool/")){
    //ool element - do nothing

  } else {
    int stackLength=stack.size();

    //if the previous event was an end event, we need to check the buffer
    if (previousEndEvent) {
      //if the buffer has content, we're in mixed territory
      //just read out the buffer
      reportEndStack();

    } else { 
      //if the buffer has content, we need to mesh it

      if (stackLength==2){
        //mesh
        reportMeshedStack();
      } else {
        //for now, just  bail
        reportEndStack();
      }
    }
   
    previousEndEvent=true;

//  reportEndElement(uri, localName, qName);  
  }

}   

protected void reportStack() throws SAXException {
      boolean pastFirstOne=false;
      while (stack.size()>0) {
         StartTag report=stack.popBottom();
         reportStartElement(report.getUri(), report.getLocalName(), report.getQName(), report.getAtts());  
         if (pastFirstOne) {
           reportEndElement(report.getUri(), report.getLocalName(), report.getQName() );  
         } else {
         pastFirstOne=true;         
         }
      }       

}

protected void reportEndStack() throws SAXException {
      while (stack.size()>0) {
         StartTag report=stack.popBottom();
         reportStartElement(report.getUri(), report.getLocalName(), report.getQName(), report.getAtts());         
         reportEndElement(report.getUri(), report.getLocalName(), report.getQName() );  
      }       

}

protected void reportMeshedStack() throws SAXException{
//     reportStack();
     StartTag report=stack.popBottom();
     Attributes currentAtts=report.getAtts();
     
     
     AttributesImpl newAtts=new AttributesImpl();

    int numAtts=currentAtts.getLength();

    for (int i=0; i<numAtts; i++) {
     newAtts.addAttribute(
        currentAtts.getURI(i),
        currentAtts.getLocalName(i),
        currentAtts.getQName(i),
        currentAtts.getType(i),
        currentAtts.getValue(i)
      );
    } //end for

     StartTag oolTag=stack.popBottom();
     Attributes oolAtts=oolTag.getAtts();

    numAtts=oolAtts.getLength();

////System.out.println("Num atts: "+numAtts);

    for (int i=0; i<numAtts; i++){
      newAtts.addAttribute(
        oolAtts.getURI(i),
        oolAtts.getLocalName(i),
        oolAtts.getQName(i),
        oolAtts.getType(i),
        oolAtts.getValue(i)
      );
    }
    
    reportStartElement(report.getUri(), report.getLocalName(), report.getQName(), newAtts);         
    reportEndElement(report.getUri(), report.getLocalName(), report.getQName() );  
}

protected void reportStartElement(String uri, String localName, String qName, Attributes atts) throws SAXException {

   StringBuffer startTag=new StringBuffer();
   
   startTag.append("<" + qName + " ");

    int numAtts=atts.getLength();

    for (int i=0; i<numAtts; i++) {
        startTag.append(atts.getQName(i)+"=");
        startTag.append("'"+atts.getValue(i)+"'");
    } //end for
   
   startTag.append(" stackSize='"); 
   
   Integer stackSize=new Integer(stack.size());
   
   startTag.append(stackSize.toString());
   
   startTag.append("'");
    
   startTag.append(">");
   
   System.out.println(startTag);
    super.startElement(uri, localName, qName, atts);

}

protected void reportEndElement(String uri, String localName, String qName) throws SAXException {
   StringBuffer endTag=new StringBuffer();
   
   endTag.append("</" + qName + ">");
   System.out.println(endTag);
   
    super.endElement(uri, localName, qName);  

}

//--------------------------------------------------------
//--------------------------------------------------------
//--------------------------------------------------------

public static void printHelp() {
    System.out.println("Mixed requires two arguments - an origin file and a  destination file.");
    System.out.println("");
    System.out.println("Options may appear before those arguments:");
    System.out.println("-p [parser class name] - specifies a SAX2 parser for use with this tool");
    System.out.println("-v - verbose output");
    System.out.println("More information is available at http://simonstl.com/projects/ool/");
    System.out.println("");
/*
    System.out.println("");
    System.out.println("");
    System.out.println("");
    System.out.println("");
*/
}

/**
FilterTester's main method requires one argument - a target XML file.

Options may appear before those arguments:<br />
-p [parser class name]  - use the parser specified<br />


*/

  // Main Method
  public static void main (String[] args) throws Exception {

    int numArgs=args.length;

    String targetFileName;
    String originFileName;
    boolean suppressOol=true;

//deal with System properties
   if ((System.getProperty("org.xml.sax.driver"))==null) {
      System.setProperty("org.xml.sax.driver","gnu.xml.aelfred2.SAXDriver");
   }


   String parserName=System.getProperty("org.xml.sax.driver");

    if (numArgs<2) {
      printHelp();
    } else {

       targetFileName=args[numArgs-1];
       originFileName=args[numArgs-2];

for (int i=0; i<numArgs; i++) {
   if (args[i].equals ("-p")) {
       parserName=args[i+1];
       i++;
   }
   if (args[i].equals ("-v")) {
       suppressOol=false;
   }

}
       //  Create files
       
       
       //  Create a SAX Parser
        XMLReader parser = XMLReaderFactory.createXMLReader(parserName); 
        
        OolCleanSAXFilter oolCleanFilter=new OolCleanSAXFilter(parser);


        FileWriter target=new FileWriter(targetFileName);  
        XMLWriter w = new XMLWriter(oolCleanFilter, target);
        FileReader origin=new FileReader(originFileName);
	    w.parse(new InputSource(origin));
      
      origin.close();
      target.close();


      
    }
  }


}
