package com.simonstl.ool;

import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;
import java.io.*;
import java.util.*;
import com.megginson.sax.XMLWriter;
import com.megginson.sax.DataWriter;

import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.ParserAdapter;
import org.xml.sax.helpers.XMLReaderFactory;

/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

The Original Code is available at  http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions  created by Simon St.Laurent are Copyright (C) 2001. All Rights Reserved. 

Contributor(s): 
*/

/**
* <p>This class provides a command-line interface to OolRemeshSAXFilter.</p>

* <p>This class just takes two arguments - a rules file and a target file.
* It loads the rules, feeds them to the filter, and sets up an XMLWriter
* so that testers can see the results on the command line.</p>
*
* <p>This class requires David Megginson's XMLWriter class, available from:
*
* @version 0.01 12 April 2002
* @author Simon St.Laurent
*/

public class OolRemesh {
    
    
/**
printHelp() lists options if the user doesn't specify any files.
*/

public static void printHelp() {
    System.out.println("OolRemesh requires three arguments - a destination file, a markup file, and a text file.");
    System.out.println("");
    System.out.println("Options may appear before those arguments:");
    System.out.println("-p [parser class name] - specifies a SAX2 parser for use with this tool");
    System.out.println("-v - verbose output");
    System.out.println("More information is available at http://simonstl.com/projects/ool/");
    System.out.println("");
/*
    System.out.println("");
    System.out.println("");
    System.out.println("");
    System.out.println("");
*/
}

/**
FilterTester's main method requires one argument - a target XML file.

Options may appear before those arguments:<br />
-p [parser class name]  - use the parser specified<br />


*/

  // Main Method
  public static void main (String[] args) throws Exception {

    int numArgs=args.length;

    String targetFileName;
    String markupFileName;
    String textFileName;
    boolean suppressOol=true;

//deal with System properties
   if ((System.getProperty("org.xml.sax.driver"))==null) {
      System.setProperty("org.xml.sax.driver","org.apache.xerces.parsers.SAXParser");
   }


   String parserName=System.getProperty("org.xml.sax.driver");

    if (numArgs<3) {
      printHelp();
    } else {

       targetFileName=args[numArgs-3];
       markupFileName=args[numArgs-2];
       textFileName=args[numArgs-1];

for (int i=0; i<numArgs; i++) {
   if (args[i].equals ("-p")) {
       parserName=args[i+1];
       i++;
   }
   if (args[i].equals ("-v")) {
       suppressOol=false;
   }

}
       //  Create files
       
       File markupFile=new File (markupFileName);
       File textFile=new File (textFileName);
       
       //  Create a SAX Parser
        XMLReader parser = XMLReaderFactory.createXMLReader(parserName); 
        
        OolRemeshSAXFilter oolRemeshFilter=new OolRemeshSAXFilter(parser);

        oolRemeshFilter.setSuppressOol(suppressOol);

        FileReader text=new FileReader(textFile);
        oolRemeshFilter.setText(text);

        FileWriter target=new FileWriter(targetFileName);  
        XMLWriter w = new XMLWriter(oolRemeshFilter, target);
        FileReader markup=new FileReader(markupFileName);
	    w.parse(new InputSource(markup));
      
      text.close();
      target.close();
      markup.close();

      System.out.println(text.toString());
      
    }
  }
}