/*
The contents of this file are subject to the Mozilla Public License  Version 1.1 (the "License"); you may not use this file except in  compliance with the License. You may obtain a copy of the License at 
http://www.mozilla.org/MPL/ 
Software distributed under the License is distributed on an "AS IS"  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  License for the specific language governing rights and limitations under  the License. 

http://simonstl.com/projects/fragment/original. 

The Initial Developer of the Original Code is Simon St.Laurent. Portions created by Simon St.Laurent are Copyright (C) 2000-2002. All Rights Reserved. 

Contributor(s): 
*/

package com.simonstl.ool;


import org.xml.sax.*;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.XMLFilter;
import org.xml.sax.helpers.XMLFilterImpl;

import java.util.*;
import java.io.*;



/**
 * <p>This class converts XML documents which use ool elements and attributes in 
 * place of embedded markup back into ordinary XML documents.</p>
 * 
 * 
 * @author Simon St.Laurent
 * @version 0.01 $Date: 2002/04/12 $
 */
public class OolRemeshSAXFilter extends XMLFilterImpl  
{
  char[] text;
  boolean suppressOol=true;
/**
* A constructor that takes the parser which will feed it SAX events
*/
public OolRemeshSAXFilter (XMLReader parent) {
      super(parent); 
} 

/**
  By default, the OolRemeshSAXFilter removes all the ool elements and attributes.  If you want to keep them, set this to false.
*/

public void setSuppressOol(boolean _suppressOol) {
    suppressOol=_suppressOol;
}

/**
  Lets you see whether or not the oolRemeshSAXFilter is going to suppress (true) or report (false) ool elements and attributes.
*/
public boolean getSuppressOol(){
   return suppressOol;
}

/**
  Sets the text the OolSAXRemeshFilter uses to fill the text back in.  This will eventually be replaced by use of the ool:src attribute.
*/
public void setText (Reader reader) throws IOException {
  StringBuffer buffer=new StringBuffer();
  char[] chunk=new char[1024];
  int numChars=1;
    while (numChars>0) {  
     numChars=reader.read(chunk);
     if (numChars>0) {
        buffer.append(chunk, 0, numChars); 
     }
   }
   
   text=buffer.toString().toCharArray();
   
}

/**
  Does most of the work of the filter, adding Characters events where necessary to fill in information represented by ool elements and attributes.
*/
public void startElement (String uri, String localName, String qName, Attributes atts) throws SAXException {
   boolean oolish=false;
   int start=0;
   int end=0;

   String startVal=atts.getValue("http://simonstl.com/ns/ool/","start");
   if (startVal!=null){
      start=Integer.parseInt(startVal);
   }

   String endVal=atts.getValue("http://simonstl.com/ns/ool/","end"); 
   if (endVal!=null){
      end=Integer.parseInt(endVal);
   }
   
   if ((startVal!=null) && (endVal!=null)) {
     oolish=true;
   }

   if (suppressOol) {
    AttributesImpl newAtts=new AttributesImpl();
    int numAtts=atts.getLength();
    for (int i=0; i<numAtts; i++) {
      if(!( atts.getURI(i).equals("http://simonstl.com/ns/ool/") )) {
       newAtts.addAttribute(
          atts.getURI(i),
          atts.getLocalName(i),
          atts.getQName(i),
          atts.getType(i),
          atts.getValue(i)
        );
      }
    } //end for
    atts=newAtts;
   }

   if ((uri.equals("http://simonstl.com/ns/ool/")) && suppressOol) {
   } else {
     super.startElement(uri, localName, qName, atts);
   }
   
   if (oolish) {
     super.characters(text, start, end-start);
   }
}

/**
  Suppresses the end tags of ool elements if necessary.
*/
public void endElement (String uri, String localName, String qName) throws SAXException {
   if ((uri.equals("http://simonstl.com/ns/ool/")) && suppressOol) {
   } else {
     super.endElement(uri, localName, qName);
   }
}
   
   
}
